<?php

namespace Drupal\layout_builder_enhancements\Plugin\Block;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\layout_builder_enhancements\ViewsService;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Entity\EntityDisplayRepository;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Block\BlockBase;

/**
 * Provides a Custom block.
 *
 * @Block(
 *   id = "layout_builder_enhancements_view_block",
 *   admin_label = @Translation("View block"),
 *   category = @Translation("View Block"),
 *   deriver = "Drupal\layout_builder_enhancements\Plugin\Derivative\ViewBlock"
 * )
 */
class ViewBlock extends BlockBase implements ContainerFactoryPluginInterface, BlockPluginInterface {

  protected $manager;
  protected $entityTypeManager;
  protected $displayRepository;
  protected $inPreview = FALSE;
  protected $offset = 0;
  protected $renderer;
  protected $viewsService;
  protected $resultCount = 0;

  /**
   * Set if this block is in preview.
   */
  public function inPreview($in_preview) {
    $this->inPreview = $in_preview;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $view = $this->getView();
    $type = $view->getBaseEntityType();
    $offset = $this->getOffset();

    $build = [];
    $this->renderer->addCacheableDependency($build, $view);
    $result = [];
    if (isset($view->result[$offset])) {
      $result = [$view->result[$offset]->_entity];
    }

    if ($this->configuration['show_all']) {
      foreach ($view->result as $row) {
        $entities[] = $row->_entity;
      }
      $result = array_slice($entities, $this->offset);
    }

    if (count($result) > 0) {
      $build = $this->entityTypeManager->getViewBuilder($type->id())
        ->viewMultiple($result, isset($this->configuration['view_mode']) ? $this->configuration['view_mode'] : 'teaser');
    }
    elseif ($this->inPreview) {
      $build = ['#markup' => $this->t('No content for this row.')];
    }
    $this->resultCount = count($result);
    return $build;
  }

  /**
   * Retrieve consumend rows.
   */
  public function getResultCount() {
    return $this->resultCount;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'label_display' => FALSE,
      'view_mode' => 'default',
    ];
  }

  /**
   * Get a view.
   */
  public function getView() {
    $block_id = $this->getDerivativeId();
    $viewId = explode(':', $block_id)[0];
    $displayId = explode(':', $block_id)[1];
    $view = $this->viewsService->getView($viewId, $displayId, $this->getDerivativeId());
    return $view;
  }

  /**
   * Get a view.
   */
  public function setOffset($offset) {
    $this->offset = $offset;
  }

  /**
   * Get a view.
   */
  protected function getOffset() {
    return $this->offset;
  }

  /**
   * Get display id.
   */
  public function getDisplayId() {
    $block_id = $this->getDerivativeId();
    return explode(':', $block_id)[1];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $view = $this->getView();
    if (!$view) {
      return ['#markup' => $this->t('View does not exist')];
    }

    $form['view_mode'] = [
      '#type' => 'select',
      '#options' => $this->displayRepository->getViewModeOptions($view->getBaseEntityType()->id()),
      '#title' => $this->t('Who'),
      '#description' => $this->t('Who do you want to say hello to?'),
      '#default_value' => isset($config['view_mode']) ? $config['view_mode'] : '',
    ];

    $form['show_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show all remaining rows'),
      '#default_value' => isset($config['show_all']) ? $config['show_all'] : '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['view_mode'] = $values['view_mode'];
    $this->configuration['show_all'] = $values['show_all'];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManager $manager,
    EntityDisplayRepository $display_repository,
    Renderer $renderer,
    ViewsService $views_service
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->manager = $manager->getStorage('view');
    $this->entityTypeManager = $manager;
    $this->displayRepository = $display_repository;
    $this->renderer = $renderer;
    $this->viewsService = $views_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('renderer'),
      $container->get('layout_builder_enhancements.views')
    );
  }

}
